#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <SEGGER_RTT_Conf.h>
#include <SEGGER_RTT.h>


#define NRF_LOG_BACKEND_MAX_STRING_LENGTH   128
#define RTT_RETRY_COUNTER   1000

static char m_rtt_buffer[NRF_LOG_BACKEND_MAX_STRING_LENGTH];
static bool m_initialized   = false;

void rtt_init(void)
{
    uint32_t ret_code;

    if (m_initialized)
    {
        return;
    }

    ret_code = SEGGER_RTT_ConfigUpBuffer(
        0,
        "Normal",
        m_rtt_buffer,
        NRF_LOG_BACKEND_MAX_STRING_LENGTH,
        SEGGER_RTT_MODE_NO_BLOCK_TRIM);

    if ( ret_code != 0)
    {
        return;
    }

}

uint32_t rtt_put(uint8_t *p_buf, uint32_t len)
{
    bool ret = true;
    uint32_t idx    = 0;
    uint32_t length = len;
    uint32_t processed;
    uint32_t watchdog_counter = RTT_RETRY_COUNTER;

    do
    {
        processed = SEGGER_RTT_WriteNoLock(0, &p_buf[idx], length);
        idx += processed;
        length -= processed;
        if (processed == 0)
        {
            // If RTT is not connected then ensure that logger does not block
            watchdog_counter--;
            if (watchdog_counter == 0)
            {
                break;
            }
        }
    } while (length);

    return ret;
}
